@@ -1435,10 +1435,12 @@ struct engine_ops _gpgme_engine_ops_uiserver =
     NULL,		/* keylist_ext */
     NULL,               /* keylist_data */
     NULL,               /* keysign */
+    NULL,               /* revsig */
     NULL,               /* tofu_policy */
     uiserver_sign,
     uiserver_verify,
     NULL,		/* getauditlog */
+    NULL,               /* setexpire */
     NULL,               /* opassuan_transact */
     NULL,		/* conf_load */
     NULL,		/* conf_save */
