@@ -850,7 +850,8 @@ _gpgme_engine_op_tofu_policy (engine_t engine,
 
 gpgme_error_t
 _gpgme_engine_op_import (engine_t engine, gpgme_data_t keydata,
-                         gpgme_key_t *keyarray)
+                         gpgme_key_t *keyarray, const char *keyids[],
+                         const char *import_filter, const char *key_origin)
 {
   if (!engine)
     return gpg_error (GPG_ERR_INV_VALUE);
@@ -858,7 +859,8 @@ _gpgme_engine_op_import (engine_t engine, gpgme_data_t keydata,
   if (!engine->ops->import)
     return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
 
-  return (*engine->ops->import) (engine->engine, keydata, keyarray);
+  return (*engine->ops->import) (engine->engine, keydata, keyarray, keyids,
+                                 import_filter, key_origin);
 }
 
 
@@ -895,7 +897,8 @@ _gpgme_engine_op_keylist_ext (engine_t engine, const char *pattern[],
 
 
 gpgme_error_t
-_gpgme_engine_op_keylist_data (engine_t engine, gpgme_data_t data)
+_gpgme_engine_op_keylist_data (engine_t engine, gpgme_keylist_mode_t mode,
+			       gpgme_data_t data)
 {
   if (!engine)
     return gpg_error (GPG_ERR_INV_VALUE);
@@ -903,7 +906,7 @@ _gpgme_engine_op_keylist_data (engine_t engine, gpgme_data_t data)
   if (!engine->ops->keylist_data)
     return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
 
-  return (*engine->ops->keylist_data) (engine->engine, data);
+  return (*engine->ops->keylist_data) (engine->engine, mode, data);
 }
 
 
