@@ -146,6 +146,12 @@ keylist_status_handler (void *priv, gpgme_status_code_t code, char *args)
       err = 0;
       break;
 
+    case GPGME_STATUS_IMPORT_OK:
+    case GPGME_STATUS_IMPORT_PROBLEM:
+    case GPGME_STATUS_IMPORT_RES:
+      err = _gpgme_import_status_handler (priv, code, args);
+      break;
+
     default:
       break;
     }
@@ -417,6 +423,7 @@ parse_sec_field15 (gpgme_key_t key, gpgme_subkey_t subkey, char *field)
     {
       /* Fields starts with a hex digit; thus it is a serial number.  */
       key->secret = 1;
+      subkey->secret = 1;
       subkey->is_cardkey = 1;
       subkey->card_number = strdup (field);
       if (!subkey->card_number)
@@ -734,7 +741,7 @@ keylist_colon_handler (void *priv, char *line)
         }
 
       /* Field 18 has the compliance flags.  */
-      if (fields >= 17 && *field[17])
+      if (fields >= 18 && *field[17])
         PARSE_COMPLIANCE_FLAGS (field[17], subkey);
 
       if (fields >= 20)
@@ -817,7 +824,7 @@ keylist_colon_handler (void *priv, char *line)
         }
 
       /* Field 18 has the compliance flags.  */
-      if (fields >= 17 && *field[17])
+      if (fields >= 18 && *field[17])
         PARSE_COMPLIANCE_FLAGS (field[17], subkey);
 
       break;
@@ -967,6 +974,30 @@ keylist_colon_handler (void *priv, char *line)
       if (fields >= 7)
 	keysig->expires = _gpgme_parse_timestamp (field[6], NULL);
 
+      /* Field 8 has the trust depth and the trust value.  */
+      if (fields >= 8 && *field[7])
+        {
+          const char *trust_depth = field[7];
+          char *trust_value = strchr (field[7] + 1, ' ');
+          if (trust_value)
+            *(trust_value++) = '\0';
+          if (trust_value)
+            {
+              int depth = atoi (trust_depth);
+              int value = atoi (trust_value);
+
+              if (depth >= 1 && depth < 256)
+                keysig->trust_depth = depth;
+              if (value >= 1 && value < 256)
+                keysig->trust_value = value;
+            }
+        }
+
+      /* Field 9 has the trust signature scope (a regular expression).  */
+      if (fields >= 9)
+	if (_gpgme_decode_c_string (field[8], &keysig->trust_scope, 0))
+	  return gpg_error (GPG_ERR_ENOMEM);	/* FIXME */
+
       /* Field 11 has the signature class (eg, 0x30 means revoked).  */
       if (fields >= 11)
 	if (field[10][0] && field[10][1])
@@ -991,7 +1022,7 @@ keylist_colon_handler (void *priv, char *line)
 	return 0;
       assert (opd->tmp_keysig == key->_last_uid->_last_keysig);
 
-      if (fields >= 4)
+      if (fields >= 5)
 	{
 	  /* Field 2 has the subpacket type.  */
 	  int type = atoi (field[1]);
@@ -1101,6 +1132,10 @@ gpgme_op_keylist_start (gpgme_ctx_t ctx, const char *pattern, int secret_only)
   if (err)
     return TRACE_ERR (err);
 
+  err = _gpgme_op_import_init_result (ctx);
+  if (err)
+    return TRACE_ERR (err);
+
   _gpgme_engine_set_status_handler (ctx->engine, keylist_status_handler, ctx);
 
   err = _gpgme_engine_set_colon_line_handler (ctx->engine,
@@ -1145,6 +1180,10 @@ gpgme_op_keylist_ext_start (gpgme_ctx_t ctx, const char *pattern[],
   if (err)
     return TRACE_ERR (err);
 
+  err = _gpgme_op_import_init_result (ctx);
+  if (err)
+    return TRACE_ERR (err);
+
   _gpgme_engine_set_status_handler (ctx->engine, keylist_status_handler, ctx);
   err = _gpgme_engine_set_colon_line_handler (ctx->engine,
 					      keylist_colon_handler, ctx);
@@ -1186,13 +1225,17 @@ gpgme_op_keylist_from_data_start (gpgme_ctx_t ctx, gpgme_data_t data,
   if (err)
     return TRACE_ERR (err);
 
+  err = _gpgme_op_import_init_result (ctx);
+  if (err)
+    return TRACE_ERR (err);
+
   _gpgme_engine_set_status_handler (ctx->engine, keylist_status_handler, ctx);
   err = _gpgme_engine_set_colon_line_handler (ctx->engine,
                                               keylist_colon_handler, ctx);
   if (err)
     return TRACE_ERR (err);
 
-  err = _gpgme_engine_op_keylist_data (ctx->engine, data);
+  err = _gpgme_engine_op_keylist_data (ctx->engine, ctx->keylist_mode, data);
   return TRACE_ERR (err);
 }
 
