@@ -272,11 +272,6 @@ main (int argc, char **argv)
       int nsigs;
 
       printf ("keyid   : %s\n", key->subkeys?nonnull (key->subkeys->keyid):"?");
-      printf ("fpr     : %s\n", key->subkeys?nonnull (key->subkeys->fpr):"?");
-      if (key->subkeys && key->subkeys->keygrip)
-        printf ("grip    : %s\n", key->subkeys->keygrip);
-      if (key->subkeys && key->subkeys->curve)
-            printf ("curve   : %s\n", key->subkeys->curve);
       printf ("caps    : %s%s%s%s\n",
               key->can_encrypt? "e":"",
               key->can_sign? "s":"",
@@ -294,9 +289,7 @@ main (int argc, char **argv)
       printf ("upd     : %lu (%u)\n", key->last_update, key->origin);
 
       subkey = key->subkeys;
-      if (subkey)
-        subkey = subkey->next;
-      for (nsub=1; subkey; subkey = subkey->next, nsub++)
+      for (nsub=0; subkey; subkey = subkey->next, nsub++)
         {
           printf ("fpr   %2d: %s\n", nsub, nonnull (subkey->fpr));
           if (subkey->keygrip)
@@ -330,6 +323,8 @@ main (int argc, char **argv)
             printf ("    name: %s\n", uid->name);
           if (uid->comment)
             printf ("   cmmnt: %s\n", uid->comment);
+          if (uid->uidhash)
+            printf (" uidhash: %s\n", uid->uidhash);
           printf ("     upd: %lu (%u)\n", uid->last_update, uid->origin);
           printf ("   valid: %s\n",
                   uid->validity == GPGME_VALIDITY_UNKNOWN? "unknown":
@@ -367,6 +362,9 @@ main (int argc, char **argv)
               printf ("     created: %s\n", isotimestr(ks->timestamp));
               printf ("     expires: %s\n", isotimestr(ks->expires));
               printf ("       class: %x\n", ks->sig_class);
+              printf (" trust depth: %u\n", ks->trust_depth);
+              printf (" trust value: %u\n", ks->trust_value);
+              printf (" trust scope: %s\n", nonnull (ks->trust_scope));
             }
         }
 
